from scarfer.format.interface import FormatInterface

class TextFormatter(FormatInterface):

    def _format_file(self, f, settings):
        ret = []
        
        copyrights = settings.get('copyrights')
        licenses = settings.get('licenses')
        matches = settings.get('matches')

        if licenses:
            ret.append("{path}: {license}".format(path=f['path'], license=str(f['license']['expressions'])))
        else:
            ret.append("{path}".format(path=f['path']))
                
        if copyrights:
            for cr in f['copyrights']:
                ret.append(" * {c}".format(c=cr))
        if matches:
            for m in f['license']['matches']:
                ret.append("--- {match} --- ".format(match=m['key']))
                ret.append("{text}".format(text=m['text']))
            
        return "\n".join(ret)

    def _format_files(self, report, settings):
        ret = []
        for f in report['files']:
            ret.append(self._format_file(f, settings))
        return "\n".join(ret)

    def _format_cumulative(self, report, settings):
        ret = []
        cumulative = report['cumulative']
        ret.append(" * license: {_license}".format(_license=cumulative['license']))
        return "\n".join(ret)
        
    
    def format(self, report, settings):

        if settings.get('license_summary') or settings.get('copyright_summary'):
            res = ""
            if settings.get('license_summary'):
                license_summary = set()
                for f in report['files']:
                    for l in f['license']['expressions']:
                        license_summary.add(l)
                res += f'Licenses:\n { " AND ".join(license_summary)}\n'

            if settings.get('copyright_summary'):
                copyright_summary = set()
                for f in report['files']:
                    for l in f['copyrights']:
                        copyright_summary.add(l)
                c_list = list(copyright_summary)
                c_list.sort()
                c_string = "\n".join(c_list)
                res += f'Copyrights:\n{c_string}\n'
            return res
        elif settings.get('cumulative'):
            return self._format_cumulative(report, settings)
        else:
            results = [
                "Files:\n----------------------------",
                self._format_files(report, settings),
            ]
            return "\n".join(results)


        
        



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2021 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import RawTextHelpFormatter
import argparse
import os

from scarfer.format.factory import FormatFactory
from scarfer.scan_interface import ScanReportReader
from scarfer.scan_interface import ScanReportException
from scarfer.format.interface import Settings
from scarfer.filter_utils import create_filters
from scarfer.config import scarfer_version
from scarfer.config import scarfer_name

SCRIPT_DIR = os.path.dirname(os.path.realpath(__file__))

PROGRAM_NAME = scarfer_name
PROGRAM_DESCRIPTION = "Source code scan report file reporter reads scan reports and outputs selected information about filtered files"
PROGRAM_URL = "https://github.com/hesa/scarfer"
PROGRAM_EXAMPLES = "   Coming soon."
BUG_URL = "https://github.com/hesa/scarfer/issues"
PROGRAM_COPYRIGHT = "(c) 2021 Henrik Sandklef<hesa@sandklef.com>"
PROGRAM_LICENSE = "GPL-3.0-or-later"
PROGRAM_AUTHOR = "Henrik Sandklef"
PROGRAM_SUPPORTED_FORMATS = "   * Scancode Output Format version 1.0.0\n   * Scaancode Toolkit, version 21 and upwards\n"
PROGRAM_ATTRIBUTION = ""
PROGRAM_SEE_ALSO = ""

OUTPUT_FORMAT_JSON = "JSON"
OUTPUT_FORMAT_TEXT = "text"
OUTPUT_FORMAT_YAML = "text"
OUTPUT_FORMATS = [ OUTPUT_FORMAT_JSON, OUTPUT_FORMAT_TEXT, OUTPUT_FORMAT_YAML ]

DATE_FMT = '%Y-%m-%d'

def parse():

    description = "NAME\n  " + PROGRAM_NAME + "\n\n"
    description = description + "DESCRIPTION\n  " + PROGRAM_DESCRIPTION + "\n\n"
    
    epilog = ""
    epilog = epilog + "EXAMPLES\n\n" + PROGRAM_EXAMPLES + "\n\n"
    epilog = epilog + "SUPPORTED SCAN REPORT FORMATS\n" + PROGRAM_SUPPORTED_FORMATS + "\n\n"
    epilog = epilog + "AUTHOR\n  " + PROGRAM_AUTHOR + "\n\n"
    epilog = epilog + "REPORTING BUGS\n  File a ticket at " + PROGRAM_URL + "\n\n"
    epilog = epilog + "COPYRIGHT\n  Copyright " + PROGRAM_COPYRIGHT + ".\n  License " + PROGRAM_LICENSE + "\n\n"
    epilog = epilog + "SEE ALSO\n  " + PROGRAM_SEE_ALSO + "\n\n"
    
    parser = argparse.ArgumentParser(
        description=description,
        epilog=epilog,
        formatter_class=RawTextHelpFormatter
    )

    parser.add_argument('file',
                            type=str,
                            help='Scan report to use',
                            default=None)
    
    parser.add_argument('-m', '--matched-text',
                            action='store_true',
                            help='output information about license matches',
                            default=False)
    
    parser.add_argument('-c', '--copyrights',
                            action='store_true',
                            help='output information about copyrights',
                            default=False)
    
    parser.add_argument('-cu', '--cumulative',
                            action='store_true',
                            help='outpur cumulative license information',
                            default=False)
    
    parser.add_argument('-l', '--license',
                            action='store_true',
                            help='output information about license',
                            default=False)
    
    parser.add_argument('-ls', '--license-summary',
                            action='store_true',
                            help='output license summary',
                            default=False)
    
    parser.add_argument('-cs', '--copyright-summary',
                            action='store_true',
                            help='output copyright summary',
                            default=False)
    
    parser.add_argument('-il', '--include-license',
                            type=str,
                            nargs="+",
                            action='append',
                            help='filter on licenses containing argument',
                            default=[])
    
    parser.add_argument('-el', '--exclude-license',
                            type=str,
                            action='append',
                            nargs="+",
                            help='filter out licenses containing argument',
                            default=[])
    
    parser.add_argument('-if', '--include-file',
                            type=str,
                            action='append',
                            nargs="+",
                            help='filter on file containing argument',
                            default=[])
    
    parser.add_argument('-ef', '--exclude-file',
                            type=str,
                            action='append',
                            nargs="+",
                            help='filter out on file containing argument',
                            default=[])
    
    parser.add_argument('-iff', '--include-file-file',
                            type=str,
                            action='append',
                            nargs="+",
                            help='filter on file containing file filters',
                            default=[])
    
    parser.add_argument('-f', '--format',
                            type=str,
                            help='output result in specified format, default is ' + OUTPUT_FORMAT_TEXT,
                            default=OUTPUT_FORMAT_TEXT)

    parser.add_argument('--version', '-V', action='version', version="{name}: {version}".format(name=scarfer_name, version=scarfer_version))

    parser.add_argument('-v', '--verbose',
                            action='store_true',
                            help='output verbose information to stderr',
                            default=False)
    args = parser.parse_args()

    return args


def _merge_file_filters(include_file, include_file_file):
    new_filters = []

    for include_list in include_file:
        for filter in include_list:
            new_filters.append(filter)

    for file_name_list in include_file_file:
        for file_name in file_name_list:
            f = open(file_name, 'r')
            for line in f.readlines():
                stripped_line = line.strip().replace("\n","")
                if not stripped_line.startswith("#") and len(stripped_line) > 0:
                    reg_exp = stripped_line
                    if not stripped_line.endswith("/"):
                        reg_exp = stripped_line + "$"
                    new_filters.append(reg_exp)
 
    return new_filters

def flatten_lists(lists):
    new_list = []
    for l in lists:
        for elem in l:
            new_list.append(elem)
    return new_list

def main():
    args = parse()

    # Create scan report reader 
    reader = ScanReportReader(args.file)

    include_license = flatten_lists(args.include_license)
    exclude_license = flatten_lists(args.exclude_license)
    exclude_file = flatten_lists(args.exclude_file)
    
    # Create filters
    include_files = _merge_file_filters(args.include_file, args.include_file_file)
    filters = create_filters(include_license, include_files)
    
    exclude_filters = create_filters(exclude_license, exclude_file)
    
    # Create output formatter
    formatter = FormatFactory.formatter(args.format)

    # Create settings map, to pass to reader (apply_filter)
    settings = Settings(args.copyrights, args.license, args.matched_text, args.cumulative, args.license_summary, args.copyright_summary)

    # Read report
    reader.read()

    # Filter the data in the report, with the filters
    reader.apply_filters(filters, exclude_filters)
    filtered_files = reader.report()

    # Frmat the data 
    formatted_data = formatter.format(filtered_files, settings)

    # Print the data
    print(formatted_data)

if __name__ == '__main__':
    main()
    



from enum import Enum
import re
from license_expression import Licensing, LicenseSymbol

from scarfer.format.factory import FormatFactory
#from format import FormatInterface


# expected result from read(), from the implementing classes
# ----------------------
# files: [
#   {
#     "path":       "tmp/tmp/apa.c",
#     "copyrights": [ "(c) 2009 Some One", "(c) 2001 Someone Elsie" ],
#     "license":    {
#       "expression": [ "mit or x11", "bsd" ],
#       "matches": [
#         {
#           "key":  "mit or x11",
#           "text": "This library is license under mit or x11"
#         }
#       ]
#     }
#  ]

#
# what to expect from report() - as a user
#
# {
#   "files": (see above)
#   "meta": {}
#   "cumulative": {
#     "license": "simplified license of all the combined"
#     "copytights": "combined copyrights"
#   }


class ScanReportFilterOperator(Enum):
    AND = 1
    OR = 2

    def __str__(self):
        return str(self.value)
    
class ScanReportFilterType(Enum):
    FILE = 1
    LICENSE = 2
    
    def __str__(self):
        return str(self.value)

class ScanReportFilter:

    def __init__(self, _expr, _type = ScanReportFilterType.FILE):
        self.expr = _expr
        self.type = _type

    def __str__(self):
        return "expr: {_expr}, type: {_type}, ".format(
            _expr = self.expr,
            _type = self.type)

            
class ScanReportReader:
    def __init__(self, file_name):
        self.file_name = file_name

    def _apply_filter_file(self, filt, f):
        if filt.type == ScanReportFilterType.FILE:
            ret = re.search(filt.expr, f['path'])
            return ret != None
        elif filt.type == ScanReportFilterType.LICENSE:
            for l in f['license']['expressions']:
                if re.search(filt.expr,l):
                    return True
            return False
        else:
            raise(ScanReportException("Unsupported filter type. This is weird."))
            

    def _apply_filters_file(self, f, filters, filter_on_files):
        
        if len(filters) == 0:
            return True
        
        ret  = False
        
        for filt in filters:
            if filter_on_files:
                if filt.type == ScanReportFilterType.FILE:
                    ret = ret or self._apply_filter_file(filt, f)
                    
            else:
                if filt.type == ScanReportFilterType.LICENSE:
                    ret = ret or self._apply_filter_file(filt, f)

        return ret

    def _generic_filter_count(self, filters, filter_type):
        count = 0
        for filt in filters:
            if filt.type == filter_type:
                count += 1
        return count
        
    def _file_filter_count(self, filters):
        return self._generic_filter_count(filters, ScanReportFilterType.FILE)
    
    def _license_filter_count(self, filters):
        return self._generic_filter_count(filters, ScanReportFilterType.LICENSE)
    
    def apply_filters(self, filters=[], exclude_filters=[]):

        # filter in on files
        if self._file_filter_count(filters) > 0:
            keep_data = []
            for f in self.data:
                #print("f: " + str(f['path']))
                if not self._apply_filters_file(f, filters, True):
                    pass
                else:
                    keep_data.append(f)
        else:
            keep_data = self.data
            
        # filter in on licenses
        if self._license_filter_count(filters) > 0:
            ret_data = []
            for f in keep_data:
                if not self._apply_filters_file(f, filters, False):
                    pass
                else:
                    ret_data.append(f)
            keep_data = ret_data

        # filter out on files
        if self._file_filter_count(exclude_filters) > 0:
            ret_data = []
            for f in keep_data:
                if not self._apply_filters_file(f, exclude_filters, True):
                    ret_data.append(f)
                else:
                    pass
            keep_data = ret_data
            
        # filter out on licenses
        if self._license_filter_count(exclude_filters) > 0:
            ret_data = []
            for f in keep_data:
                if not self._apply_filters_file(f, exclude_filters, False):
                    ret_data.append(f)
                else:
                    pass
            keep_data = ret_data
                    

        licenses = set()
        for f in keep_data:
            for lic in f['license']['expressions']:
                licenses.add(lic)

        license_string = " and ".join(licenses)
        licensing = Licensing()
        parsed = licensing.parse(license_string)

        report_data = {}
        report_data['files'] = keep_data
        report_data['meta'] = {}
        report_data['cumulative'] = {}
        report_data['cumulative']['license'] = parsed
                
        self.report_data = report_data
    
    def read(self):
        clazzes = [ FakeReportReader, FakeReportReader, ScancodeReportReader ]
        self.data = None
        for clazz in clazzes:
            try:
                self.data = clazz(self.file_name).read()
                break
            except Exception as e:
                pass
        if self.data == None:
            raise(ScanReportException(f"File {self.file_name} not in a supported format"))
        
    def report(self):
        if self.report_data == None:
            self.apply_filters([])
        return self.report_data

    def raw_report(self):
        return self.data
    
    def __str__(self):
        ret = ["path: {name}".format(name=self.file_name)]
#        ret.append("operator: {operator}".format(operator=self.operator))
#        for f in filters:
#            ret.append("  {filter}".format(filter=f))
        return '\n'.join(ret)
            
class ScanReportException(Exception):

    def __init__(self, message=""):
        self.message = message
        super().__init__(self.message)

class FakeReportReader(ScanReportReader):

    def read(self):
        raise(ScanReportException("File not in fake format"))
        
class ScancodeReportReader(ScanReportReader):

    def check_file_format(self):
        headers = self.json_data['headers'][0]
        #print(str(headers))
        tool = headers['tool_name']
        self.scancode_format = headers.get('output_format_version', '')
        #print("tool: " + str(tool))
        if tool.lower() != "scancode-toolkit":
            raise(ScanReportException("File not in Scancode format"))

        if self.scancode_format == "2.0.0":
            self.copyright_value = "copyright"
        else:
            self.copyright_value = "value"
        
    
    def read(self):
        with open(self.file_name) as fp:
            import json
            self.json_data = json.load(fp)
            self.check_file_format()
            self.files = self.json_data['files']

        files = []
        for f in self.files:
            
            if not f['type'] == "file":
                continue
            
            _file = {}
            
            # path
            _file['path'] = f['path']
            
            # copyright
            _file['copyrights'] = []
            for c in f['copyrights']:
                #print(" c: " + str(c) + "  reading: " + self.copyright_value)
                _file['copyrights'].append(c[self.copyright_value])
                pass
            _file['license'] = []

            # license
            _file['license'] = {}
            matches = []
            for l in f['licenses']:
                matches.append({
                    "key": l['key'],
                    "text": l['matched_text']
                    })
            _file['license']['expressions'] = f['license_expressions']
            _file['license']['matches'] = matches

                
            files.append(_file)
            
        return files

